/*
 * (C) Copyright 2006-2007, Axis Communications AB, LUND, SWEDEN
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#define _GNU_SOURCE
#include <sys/ioctl.h>
#include <net/if.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/sockios.h>
#ifdef __ARM__
#include <asm/arch/ethernet.h>
#else
#include <asm/ethernet.h>
#endif

int main(int argc, char* const argv[]) {
  int eth_fd;
  FILE* f;
  char* line;
  char value[80];
  int size;
  char* ifname = "eth0";
  int opt;
  char conf_file[40 + IF_NAMESIZE];
  struct ifreq ifr;

  while (-1 < (opt = getopt(argc, argv, "i:")))
  {
    switch ((char)opt)
    {
     case 'i':
      ifname = optarg;
      if (IF_NAMESIZE < strlen(ifname))
      {
        fprintf(stderr, "bad if name \"%s\"\n", ifname);
        exit(EXIT_FAILURE);
      }
    }
  }

  memset(&ifr, 0, sizeof(ifr));
  strcpy(ifr.ifr_name, ifname);

  line = malloc(80);
  (void)snprintf(conf_file, sizeof(conf_file), "/etc/conf.d/net.%s", ifname);
  f = fopen(conf_file, "r");
  if (!f)
  {
    perror("eth_ctrl: can't open config file");
    exit (EXIT_FAILURE);
  }
  while (!feof(f))
  {
    size = 80;
    getline(&line, &size, f);
    if (sscanf(line, "AUTONEG=%s", value))
    {
      ifr.ifr_data = (void*)1;
      eth_fd = socket(AF_INET, SOCK_DGRAM, 0);
      if (!strcasecmp(value, "never"))
	ifr.ifr_data = (void*)0;
      else if (!strcasecmp(value, "normal"))
	ifr.ifr_data = (void*)1;
      ioctl(eth_fd, SET_ETH_AUTONEG, &ifr);
      close(eth_fd);
      break;
    }
  }
  return 0;
}
